#!/bin/bash

# This script  is install by unitrends-elk rpm and it is added to crontab to run once a day to check if asset should be enable to send logs to ELK
# Author: <agonzalez@unitrends.com>

function retry()
{
        local n=0
        $@
        while [ $? -ne 0 -a $n -lt 5 ]
        do
                sleep 2
                ((n++))
                echo "retying $n"
                $@
        done
}

start_filebeat()
{
        echo
        echo [*] Configuring Filebeat/Metricbeat...

        /bin/cp -f /usr/bp/bin/elk/filebeat /etc/init.d/
        chmod a+x /etc/init.d/filebeat

        # FILEBEAT config file
        filebeat=`echo $result|grep "FILEBEAT_FILE="|cut -d= -f2|tr -d '\n'|tr -d '\r'`
        if [ ! -z $filebeat ]
        then
                echo "  - Downloading custom filebeat config: $filebeat:"
                download=$filebeat
        else
                echo "  - Downloading default filebeat config: filebeat.yml"
                download=filebeat.yml
        fi

        retry curl -s $BASEURL$download -o /tmp/$download

        grep "es.telemetry.unitrends.com" "/tmp/$download" > /dev/null

        if [ $? -eq 0 ]
        then
                echo "  - Replacing /etc/filebeat/filebeat.yml"
                mv -v /tmp/$download /etc/filebeat/
        else
                echo "  [ERROR] downloaded file is not valid"
        fi
        rm -fr /tmp/$download

        echo "[*] Starting filebeat service..."
        /sbin/chkconfig --add filebeat
        /etc/init.d/filebeat restart 2> /dev/null
}

start_metricbeat()
{
        /bin/cp -f /usr/bp/bin/elk/metricbeat /etc/init.d/
        chmod a+x /etc/init.d/metricbeat

        metricbeat=`echo $result|grep "METRICBEAT_FILE="|cut -d= -f2|tr -d '\n'|tr -d '\r'`
        if [ ! -z $metricbeat ]
        then
                echo "  - Downloading custom metricbeat config: $metricbeat:"
                download=$metricbeat
        else
                echo "  - Downloading default metricbeat config: metricbeat.yml"
                download=metricbeat.yml
        fi

        retry curl -s $BASEURL$download -o /tmp/$download

        grep "es.telemetry.unitrends.com" "/tmp/$download" > /dev/null

        if [ $? -eq 0 ]
        then
                echo "  - Replacing /etc/metricbeat/metricbeat.yml"
                mv -v /tmp/$download /etc/metricbeat/
        else
                echo "  [ERROR] downloaded file is not valid"
        fi

        rm -fr /tmp/$download

        echo "[*] Starting metricbeat service..."
        /sbin/chkconfig --add metricbeat
        /etc/init.d/metricbeat restart 2> /dev/null
}

start() {
        enable_filebeat=`echo $result|grep "FILEBEAT=true"`
        if [ ! -z $enable_filebeat ]
        then
                start_filebeat
        fi

        enable_metricbeat=`echo $result|grep "METRICBEAT=true"`
        if [ ! -z $enable_metricbeat ]
        then
                start_metricbeat
        fi

        echo "[*] Adding ELK as SNMP target..."
        echo '{"destination":"ls.telemetry.unitrends.com","enabled":1}'|curl -s -d @- http://localhost/api/traps/destinations/?sid=1

        echo "[*] Enabling elkcollect schedule..."
        ( crontab -l | grep -v -F "elkcollect.exe" ) | crontab -

         /bin/cp -f /usr/bp/bin/elk/elkcollect.cron /etc/cron.d/unitrends-elkcollect
         /bin/cp -f /usr/bp/bin/elk/elk.cron /etc/cron.d/unitrends-elk

         # Enable triggers
        psql -c "ALTER TABLE bp.jobs ENABLE TRIGGER elk_job_finished" > /dev/null 2>&1
        psql -c "ALTER TABLE bp.alerts ENABLE TRIGGER elk_alert" > /dev/null 2>&1

#       echo "[*] Reschedule elk.sh..."
#       ( crontab -l | grep -v -F "/usr/bp/bin/elk/elk.sh" ; echo "30 23 * * * /usr/bp/bin/elk/elk.sh delay &>> /var/log/unitrends-elk.log" ) | crontab -

}

stop() {
        crontab -l | grep -v "elkcollect.exe" | grep -v "elk.sh" | crontab -
        rm -fr /etc/cron.d/unitrends-elkcollect
        rm -fr /etc/cron.d/unitrends-elk
        psql -c "ALTER TABLE bp.jobs DISABLE TRIGGER elk_job_finished" > /dev/null 2>&1
        psql -c "ALTER TABLE bp.alerts DISABLE TRIGGER elk_alert" > /dev/null 2>&1
        /etc/init.d/filebeat stop 2> /dev/null
        /etc/init.d/metricbeat stop 2> /dev/null
        /sbin/chkconfig --del filebeat
        /sbin/chkconfig --del metricbeat
        rm -fr /var/lib/filebeat/registry
}

disable() {
        stop
        touch /usr/bp/bin/elk/elk.disabled
        echo "[DISABLED] - ELK has been disabled."
}

status() {
        if [ -f /usr/bp/bin/elk/elk.disabled ]
        then
                echo "[DISABLED] - ELK is disabled."
        else
                echo "[ENABLED] - ELK is enabled."
        fi
}

enable() {
        start
        rm -fr /usr/bp/bin/elk/elk.disabled
}

query() {
        if [ -f /usr/bp/bin/elk/elk.disabled ]
        then
                echo
                echo "[DISABLED] - ELK is disabled, run /usr/bp/bin/elk.sh enable to enable it again."
                return
        fi

        echo [*] Checking ELK service for asset $ASSET_TAG ...
        retry curl -s $URL$ASSET_TAG -o /tmp/elk_curl_output
        result=`cat /tmp/elk_curl_output 2> /dev/null`
        rm -fr /tmp/elk_curl_output

        echo [*] service response:
        echo     $result
        echo

        action=`echo $result|grep "ACTION="|cut -d= -f2|tr -d '\n'|tr -d '\r'`

        echo "----- Processing action: $action ------"

        case "$action" in
                START)
                        start
                ;;
                STOP)
                        stop
                ;;
                RESET)
                        stop
                        start
                ;;
                *)
                        echo "[*] Nothing to do on this appliance."
                        exit 1
                ;;
        esac
}

update()
{
        echo "[*] Checking for new version ..."
        TMP_FILE="/tmp/elk_latest_version"
        retry curl -s http://api.telemetry.unitrends.com/unitrends-elk-bin/latest_version.txt -o $TMP_FILE
        remote_version=`cat $TMP_FILE 2>/dev/null`
        local_version=`mono /usr/bp/bin/elk/elkcollect.exe version`

        if [ $? -ne 0 ]
        then
                local_version="unknown"
        fi

        rm -fr $TMP_FILE

        if [ -z "$remote_version" ]
        then
                echo "  - Cant check local or remote version"
                return
        fi

        if [ $local_version != $remote_version ]
        then
                echo "  - New version found:"
                echo "    - Installed: $local_version"
                echo "    - Available: $remote_version"
                echo "  - Downloading ..."
                retry curl -s http://api.telemetry.unitrends.com/unitrends-elk-bin/unitrends-elk-bin.zip -o /tmp/unitrends-elk-bin.zip
                echo "  - Extracting ..."
                unzip -o /tmp/unitrends-elk-bin.zip -d /usr/bp/bin/elk/
                chmod a+x /usr/bp/bin/elk/elkcollect.exe
                chmod a+x /usr/bp/bin/elk/elk.sh
                chmod a+x /usr/bp/bin/elk/elk_job_trigger.sh
                chmod a+x /usr/bp/bin/elk/elk_alert_trigger.sh
                rm -fr /tmp/unitrends-elk-bin.zip
                /bin/cp -f /usr/bp/bin/elk/elkcollect.cron /etc/cron.d/unitrends-elkcollect
                /bin/cp -f /usr/bp/bin/elk/elk.cron /etc/cron.d/unitrends-elk
        fi
}

run_script()
{
        echo "[*] Running script..."
        /usr/local/bin/dpu version|grep "10.0.0-"
        if [ $? -eq 0 ]
        then
                if [ -f /usr/bp/bin/elk/jobs-status-from-ID.php ]
                then
                        echo "  - Replacing jobs-status-from-ID.php"
                        cp -fr /usr/bp/bin/elk/jobs-status-from-ID.php /var/www/html/api/includes/jobs/jobs-status-from-ID.php
                fi
        fi
}

delay()
{
        echo ""
        echo "===[ `date`] ============================================================="
        i=$(( ( RANDOM % 1200 )  + 1 ))
        echo "[*] Waiting $i seconds ..."
        sleep $i
}


IFS=
BASEURL="http://api.telemetry.unitrends.com/"
URL="http://api.telemetry.unitrends.com/api/asset/"
ASSET_TAG=`cat /etc/unitrends-asset 2>/dev/null`
croncmd="mono /usr/bp/bin/elk/elkcollect.exe &>> /var/log/unitrends-elkcollect.log"
croncmd2="mono /usr/bp/bin/elk/elkcollect.exe level1 &>> /var/log/unitrends-elkcollect.log"
cronjob="55 23 * * * $croncmd"
cronjob2="0 */4 * * * $croncmd2"

# kill existing elkcollect in case it hung
pgrep elkcollect.exe 2>/dev/null|xargs kill -9 2>/dev/null

# purge logs
log_size=$(du -sb /var/log/unitrends-elk.log 2>/dev/null | awk 'END { if (!NR) {print "0"} else {print $1} }')
if [ "$log_size" -gt 300000 ]
then
        echo > /var/log/unitrends-elk.log
        echo "elk log purged"
fi

log_size=$(du -sb /var/log/unitrends-elkcollect.log 2>/dev/null | awk 'END { if (!NR) {print "0"} else {print $1} }')
if [ "$log_size" -gt 300000 ]
then
        echo > /var/log/unitrends-elkcollect.log
        echo "elk-collect purged"
fi


# kill existing elkcollect in case it hung
pgrep elkcollect.exe 2>/dev/null|xargs kill -9 2>/dev/null

case "$1" in
        stop)
                stop
        ;;
        update)
                update
        ;;
        delay)
                delay
                update
                run_script
                query
        ;;
        disable)
                disable
        ;;
        enable)
                enable
        ;;
        status)
                status
        ;;
        *)
                update
                run_script
                query
        ;;
esac
