#!/bin/bash

ERRORLOG="/backups/UnitrendsDataBase/pg_log/elk-alert-trigger.log"

ASSET=$(cat /etc/unitrends-asset)
dt=`date`
if [ -f $ERRORLOG ]; then
	log_size=$(du -sb $ERRORLOG | awk '{print $1}')
	if [ $log_size -gt 2000000 ]; then
		echo > $ERRORLOG
		echo -e "$dt: logfile rolled-over"
	fi
fi

while getopts "i:m:c:u:n:s:r:" flag
do
	case $flag in
		i) ALERTID=${OPTARG};;
		m) MESSAGE=${OPTARG};;
		c) CREATED=${OPTARG};;
		u) UPDATED=${OPTARG};;
		n) SNAME=${OPTARG};;
		s) SEVERITY=${OPTARG};;
		r) RESOLVED=${OPTARG};;
	esac
done

# Convert source_id to name
SNAME=`psql -tc "select source_name from bp.alert_source where source_id = $SNAME" | tr -d '[:space:]'`

# Convert severity to string
case $SEVERITY in
	1) SEVERITY="critical";;
	2) SEVERITY="warning";;
	3) SEVERITY="notice";;
esac

if [ "$RESOLVED" == "t" ]; then
	RESOLVED="true"
else
	RESOLVED="false"
fi

index="ueb-alerts-$(date +%Y.%m)"
CREATED_STR=$(date -d@$CREATED +'%d/%m/%Y %r')
UPDATED_STR=$(date -d@$UPDATED +'%d/%m/%Y %r')
TIMESTAMP=$(date --rfc-3339=seconds|tr ' ' 'T')

JSON='{
    "trigger":true,
    "created":'\"$CREATED_STR\"',
    "id":'$ALERTID',
    "message":'\"$MESSAGE\"',
    "resolved":'"$RESOLVED"',
    "severity":'\"$SEVERITY\"',
    "source_name":'\"$SNAME\"',
    "updated":'\"$UPDATED_STR\"',
    "timestamp":'\"$TIMESTAMP\"',
    "asset_tag":'\"$ASSET\"'
}'

echo $JSON|curl -k -XPOST https://unitrends:collect1@es.telemetry.unitrends.com/$index/alert/$ASSET"_"$ALERTID/?pretty -H 'Content-Type: application/json' -d @- >/dev/null 2>&1

if [ $? -ne 0 ]; then
        echo -e "${dt}: Command Failed! $JSON" >> $ERRORLOG
fi
