<?php

class JobsStatusFromID
{

    public function __construct($BP, $Functions)
    {
        $this->BP = $BP;
        $this->functions = $Functions;

        require_once('jobs.functions.php');
        $this->jobsFunctions = new JobsFunctions($this->BP);
    }


    public function get_jobs_status_from_ID($data, $sid = false)
    {
        $jobStatus = array();

        if (isset($data) and isset($data['job_type']) and isset($data['job_start_time']) and isset($data['id']) )
        {
            switch ($data['job_type'])
            {
                case Constants::JOBORDER_TYPE_ARCHIVE:
                    $archive_id = (int)$data['id'];
                    if ( $archive_id > 0 )
                    {
                        // Archive Job
                        $schedule_history_result_format = array();
                        $schedule_history_result_format['include_disabled'] = true;
                        $schedule_history_result_format['include_on_demand'] = true;

                        $schedule_history_result_format['start_time'] = (int)$data['job_start_time'];
                        $schedule_history_result_format['end_time'] = $schedule_history_result_format['start_time'];
                        //$schedule_history_result_format['schedule_id'] = (int)$data['schedule_id'];

                        $instance_id = (int)$data['instance_id'];

                        $schedule_history_result_array = $this->BP->get_schedule_history($schedule_history_result_format, $sid);

                        if ($schedule_history_result_array !== false)
                        {
                            $jobStatus['jobs'] = array();
                            foreach ($schedule_history_result_array as &$schedule)
                            {
                                foreach ( $schedule['backups'] as $backup_time )
                                {
                                    foreach ( $backup_time as $backup_from_schedule_history ) {
                                        if (isset($backup_from_schedule_history['backup_id']))
                                        {
                                            if ($archive_id === $backup_from_schedule_history['backup_id'])
                                            {
                                                $temp_backup_array = array();
                                                //$temp_backup_array = $backup_from_schedule_history;
                                                $temp_backup_array['name'] = $schedule['schedule_name'];
                                                $temp_backup_array['failures'] = isset($schedule['failures']) ? $schedule['failures'] : "";

                                                if (isset($backup_from_schedule_history['instance_id']))
                                                {
                                                    $iNames = $this->functions->getInstanceNames($backup_from_schedule_history['instance_id'], $sid);
                                                    if ($iNames !== false)
                                                    {
                                                        $temp_backup_array['asset_name'] = $iNames['asset_name'];
                                                        $temp_backup_array['client_name'] = $iNames['client_name'];
                                                        $temp_backup_array['app_name'] = $iNames['app_name'];
                                                        $temp_backup_array['app_type'] = $iNames['app_type'];

                                                    }
                                                }

                                                $temp_backup_array['instance_id'] = isset($backup_from_schedule_history['instance_id']) ? $backup_from_schedule_history['instance_id'] : "";
                                                $temp_backup_array['message'] = isset($backup_from_schedule_history['description']) ? $backup_from_schedule_history['description'] : "";
                                                $temp_backup_array['primary_name'] = isset($backup_from_schedule_history['primary_name']) ? $backup_from_schedule_history['primary_name'] : "";
                                                $temp_backup_array['secondary_name'] = isset($backup_from_schedule_history['secondary_name']) ? $backup_from_schedule_history['secondary_name'] : "";
                                                if (isset($data['job_start_time']))
                                                {
                                                    $temp_backup_array['start_time'] = strftime('%m/%d/%Y %r', $backup_from_schedule_history['job_start_time']);
                                                    $temp_backup_array['sort_start_time'] = $backup_from_schedule_history['job_start_time'];
                                                }
                                                $temp_backup_array['failed'] = isset($backup_from_schedule_history['failed']) ? $backup_from_schedule_history['failed'] : "";
                                                $temp_backup_array['mode'] = isset($backup_from_schedule_history['type']) ? $this->functions->getBackupTypeDisplayName($backup_from_schedule_history['type']) : "";
                                                $temp_backup_array['files'] = isset($backup_from_schedule_history['files']) ? $backup_from_schedule_history['files'] : "";
                                                $temp_backup_array['size'] = isset($backup_from_schedule_history['size']) ? $backup_from_schedule_history['size'] : "";
                                                $temp_backup_array['id'] = $temp_backup_array['backup_id'] = isset($backup_from_schedule_history['backup_id']) ? $backup_from_schedule_history['backup_id'] : "N/A";
                                                $temp_backup_array['status'] = $this->jobsFunctions->getBackupStatusName($temp_backup_array['failed'], $temp_backup_array['message']);
                                                $temp_backup_array['type'] = "Backup Copy";


                                                $jobStatus['jobs'][] = $temp_backup_array;
                                                break 3;  //Exit all of the schedule history loops once the archive is found
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        else
                        {
                            $jobStatus = false;
                        }

                    }
                    else
                    {
                        $jobStatus['error'] = 500;
                        $jobStatus['message'] = "Invalid input: 'id' cannot be 0 for backup copy jobs";
                    }
                    break;
                case Constants::JOBORDER_TYPE_BACKUP:
                    $backup_id = (int)$data['id'];
                    if ( $backup_id > 0 )
                    {
                        $clients = $this->BP->get_client_list($sid);

                        $backup_status_result_format = array();

                        if ($sid !== false )
                        {
                            $backup_status_result_format['system_id'] = $sid;
                        }

                        $backup_status_result_format['grandclients'] = false; //check this
                        $backup_status_result_format['backup_ids'] = array( $backup_id );


                        $backup_status_result_array = $this->BP->get_backup_status($backup_status_result_format);

                        if ($backup_status_result_array !== false) {
                            foreach ($backup_status_result_array as &$backup)
                            {
                                if ($backup['replicated'] === false)
                                {
                                    $temp_backup_array = array();
                                    $temp_backup_array['app_name'] = (array_key_exists('app_name', $backup) ? $backup['app_name'] :'');
                                    if ( array_key_exists('certified', $backup) )
                                    {
                                        switch ($backup['certified'])
                                        {
                                            case true:
                                                $temp_backup_array['certify_status'] = Constants::STATUS_SUCCESS;
                                                break;
                                            default:
                                                $temp_backup_array['certify_status'] = Constants::STATUS_FAILURE;
                                                break;
                                        }
                                    }
                                    else
                                    {
                                        $temp_backup_array['certify_status'] = 'None';
                                    }
                                    $temp_backup_array['client_name'] = $clients[$backup['client_id']];
                                    $temp_backup_array['complete'] = $backup['complete'];
                                    $temp_backup_array['database_name'] = (array_key_exists('database_name', $backup) ? $backup['database_name'] : '');
                                    $temp_backup_array['elapsed_time'] = $this->functions->formatTimeDelta($backup['elapsed_time']);
                                    $temp_backup_array['encrypted'] = $backup['encrypted'];
                                    $temp_backup_array['end_time'] = strftime('%m/%d/%Y %r', $backup['start_time'] + $backup['elapsed_time']);
                                    if( array_key_exists('files', $backup) )
                                    {
                                        $temp_backup_array['files'] = $backup['files'];
                                    }
                                    $temp_backup_array['id'] = $temp_backup_array['backup_id'] = $backup['id'];
                                    $temp_backup_array['instance_name'] = (array_key_exists('server_instance_name', $backup) ? $backup['server_instance_name'] : ((array_key_exists('sharepoint_guid', $backup) ? $backup['sharepoint_guid'] :'')));

                                    $iid = isset($backup['instance_id']) ? $backup['instance_id'] : false;
                                    if ($temp_backup_array['database_name'] == '' && $iid !== false){
                                        $instanceInfo = $this->functions->getInstanceNames($iid, $sid);
                                        if ($instanceInfo['app_type'] == Constants::APPLICATION_TYPE_NAME_ORACLE){
                                            $temp_backup_array['database_name'] = $instanceInfo['asset_name'];
                                        }
                                    }
                                    $temp_backup_array['mode'] = $this->functions->getBackupTypeDisplayName($backup['type'], $backup['synthesized']);
                                    switch ($backup['sync_status'])
                                    {
                                        case 1:
                                            $temp_backup_array['replication_status'] = Constants::STATUS_NEEDED;
                                            break;
                                        case 2:
                                            $temp_backup_array['replication_status'] = Constants::STATUS_COMPLETED;
                                            break;
                                        case 4:
                                            $temp_backup_array['replication_status'] = Constants::STATUS_IN_PROGRESS;
                                            break;
                                        case 8:
                                            $temp_backup_array['replication_status'] = Constants::STATUS_WAITING;
                                            break;
                                        default:
                                            $temp_backup_array['replication_status'] = Constants::STATUS_NOT_APPLICABLE;
                                            break;
                                    }
                                    $temp_backup_array['size'] = $backup['size'];
                                    $temp_backup_array['start_time'] = strftime('%m/%d/%Y %r', $backup['start_time']);
                                    if ( array_key_exists('status', $backup) )
                                    {
                                        switch ($backup['status'])
                                        {
                                            case 0:
                                                $temp_backup_array['status'] = Constants::STATUS_SUCCESS;
                                                break;
                                            case 1:
                                                $temp_backup_array['status'] = Constants::STATUS_WARNING;
                                                break;
                                            default:
                                                $temp_backup_array['status'] = Constants::STATUS_FAILURE;
                                                break;
                                        }
                                    }
                                    else
                                    {
                                        $temp_backup_array['status'] = Constants::STATUS_RUNNING;
                                    }
                                    $temp_backup_array['speed'] = $this->getSpeed($backup);
                                    $temp_backup_array['system_name'] = $this->functions->getSystemNameFromID($sid);
                                    $temp_backup_array['type'] = "Backup";
                                    switch ($backup['verify_status'])
                                    {
                                        case 1:
                                            $temp_backup_array['verify_status'] = Constants::STATUS_SUCCESS;
                                            break;
                                        case 2:
                                            $temp_backup_array['verify_status'] = Constants::STATUS_FAILURE;
                                            break;
                                        case 4:
                                            $temp_backup_array['verify_status'] = Constants::STATUS_RUNNING;
                                            break;
                                        default:
                                            $temp_backup_array['verify_status'] = Constants::STATUS_NOT_APPLICABLE;
                                            break;
                                    }

                                    // for restore jobs, since bp_get_schedule_history doesn't return anything, use the info available from bp_get_backup_status
                                    if ($temp_backup_array['mode'] === Constants::BACKUP_DISPLAY_TYPE_RESTORE) {
                                        if (isset($backup['instance_id'])) {
                                            $iNames = $this->functions->getInstanceNames($backup['instance_id'], $sid);
                                            if ($iNames !== false) {
                                                $temp_backup_array['asset_name'] = $iNames['asset_name'];
                                                $temp_backup_array['client_name'] = $iNames['client_name'];
                                                $temp_backup_array['app_name'] = $iNames['app_name'];
                                                $temp_backup_array['app_type'] = $iNames['app_type'];
                                            }
                                        } else if (isset($backup['client_id'])) {
                                            $temp_backup_array['asset_name'] = $temp_backup_array['client_name'];
                                        }
                                    } else {
                                        //get schedule history for non-restore jobs
                                        $schedule_history_result_format = array();
                                        $schedule_history_result_format['include_disabled'] = true;
                                        $schedule_history_result_format['include_on_demand'] = true;

                                        if (isset($data['job_start_time'])) {
                                            //UNIBP- 15845: the job start time might not be the scheduled start time as the job might have been queued(waiting for other jobs to complete)
                                            //so, setting the scheduled start time an hour before the job start time - 3600 is one hour in seconds
                                            $schedule_history_result_format['start_time'] = (int)$data['job_start_time'] - 3600;
                                            //not passing in any end time to the schedule history API
//                                        if( isset($backup['start_time']) and $backup['start_time'] > $schedule_history_result_format['start_time'] )
//                                        {
//                                            $schedule_history_result_format['end_time'] = $backup['start_time'];
//                                        }
//                                        else
//                                        {
//                                            $schedule_history_result_format['end_time'] = $schedule_history_result_format['start_time'];
//                                        }
                                        } elseif (isset($backup['start_time'])) {
                                            //If the job start time was not supplied, check through the week before the backup started
                                            //604800 is one week in seconds
                                            $schedule_history_result_format['start_time'] = $backup['start_time'] - 604800;
                                            $schedule_history_result_format['end_time'] = $backup['start_time'];
                                        }
                                        //Otherwise, look through all of schedule history to see if the backup exists.

                                        $schedule_history_result_array = $this->BP->get_schedule_history($schedule_history_result_format, $sid);

                                        $temp_backup_array['name'] = 'Job No Longer Exists';

                                        if ($schedule_history_result_array !== false) {
                                            foreach ($schedule_history_result_array as &$schedule) {
                                                foreach ($schedule['backups'] as $backup_time => $backups_at_that_time) {
                                                    foreach ($backups_at_that_time as $backup_from_schedule_history) {
                                                        if (isset($backup_from_schedule_history['backup_id'])) {
                                                            if ($backup_id === $backup_from_schedule_history['backup_id']) {
                                                                $temp_backup_array['name'] = $schedule['schedule_name'];
                                                                $temp_backup_array['failures'] = isset($schedule['failures']) ? $schedule['failures'] : "";

                                                                if (isset($backup_from_schedule_history['instance_id'])) {
                                                                    $iNames = $this->functions->getInstanceNames($backup_from_schedule_history['instance_id'], $sid);
                                                                    if ($iNames !== false) {
                                                                        $temp_backup_array['asset_name'] = $iNames['asset_name'];
                                                                        $temp_backup_array['client_name'] = $iNames['client_name'];
                                                                        $temp_backup_array['app_name'] = $iNames['app_name'];
                                                                        $temp_backup_array['app_type'] = $iNames['app_type'];
                                                                    }
                                                                }

                                                                $temp_backup_array['instance_id'] = isset($backup_from_schedule_history['instance_id']) ? $backup_from_schedule_history['instance_id'] : "";
                                                                $temp_backup_array['message'] = isset($backup_from_schedule_history['description']) ? $backup_from_schedule_history['description'] : "";
                                                                $temp_backup_array['primary_name'] = isset($backup_from_schedule_history['primary_name']) ? $backup_from_schedule_history['primary_name'] : "";
                                                                $temp_backup_array['secondary_name'] = isset($backup_from_schedule_history['secondary_name']) ? $backup_from_schedule_history['secondary_name'] : "";
                                                                /*if (isset($backup['job_start_time'])) {
                                                                    $temp_backup_array['start_time'] = strftime('%m/%d/%Y %r', $backup_from_schedule_history['job_start_time']);
                                                                    $temp_backup_array['sort_start_time'] = $backup_from_schedule_history['job_start_time'];
                                                                }*/
                                                                $temp_backup_array['failed'] = isset($backup_from_schedule_history['failed']) ? $backup_from_schedule_history['failed'] : "";
                                                                //$temp_backup_array['mode'] = isset($backup_from_schedule_history['type']) ? $this->functions->getBackupTypeDisplayName($backup_from_schedule_history['type']) : "";
                                                                $temp_backup_array['files'] = isset($backup_from_schedule_history['files']) ? $backup_from_schedule_history['files'] : "";
                                                                //$temp_backup_array['size'] = isset($backup_from_schedule_history['size']) ? $backup_from_schedule_history['size'] : "";
                                                                //$temp_backup_array['id'] = $temp_backup_array['backup_id'] = isset($backup_from_schedule_history['backup_id']) ? $backup_from_schedule_history['backup_id'] : "N/A";
                                                                //$temp_backup_array['status'] = $this->getBackupStatusName($temp_backup_array['failed'], $temp_backup_array['message']);

                                                                break 3;  //Exit all of the schedule history loops once the name is extracted
                                                            }
                                                        }
                                                    }
                                                }
                                            }// break 3 comes to here
                                        } else {
                                            global $Log;
                                            $msg = "bp_get_schedule_history failed: " . $this->BP->getError();
                                            $Log->writeError($msg, true);
                                        }
                                    }

                                    $jobStatus['jobs'] = array($temp_backup_array);
                                }
                            }
                        }
                        else
                        {
                            $jobStatus = false;
                        }
                    }
                    elseif ( isset($data['instance_id']) )
                    {
                        // Job Failed
                        $schedule_history_result_format = array();
                        $schedule_history_result_format['include_disabled'] = true;
                        $schedule_history_result_format['include_on_demand'] = true;

                        $schedule_history_result_format['start_time'] = (int)$data['job_start_time'];
                        $schedule_history_result_format['end_time'] = $schedule_history_result_format['start_time'];
                        //$schedule_history_result_format['schedule_id'] = (int)$data['schedule_id'];

                        $instance_id = (int)$data['instance_id'];

                        $schedule_history_result_array = $this->BP->get_schedule_history($schedule_history_result_format, $sid);

                        if ($schedule_history_result_array !== false)
                        {
                            $jobStatus['jobs'] = array();
                            foreach ($schedule_history_result_array as &$schedule)
                            {
                                foreach ( $schedule['backups'] as $backup_time )
                                {
                                    foreach ( $backup_time as $backup_from_schedule_history )
                                    {
                                        if ( $instance_id === $backup_from_schedule_history['instance_id'] )
                                        {
                                            $temp_backup_array = array();
                                            //$temp_backup_array = $backup_from_schedule_history;
                                            $temp_backup_array['joborder_name'] = $schedule['schedule_name'];
                                            $temp_backup_array['failures'] = isset($schedule['failures']) ? $schedule['failures'] : "";

                                            if (isset($backup_from_schedule_history['instance_id'])) {
                                                $iNames = $this->functions->getInstanceNames($backup_from_schedule_history['instance_id'], $sid);
                                                if ($iNames !== false) {
                                                    $temp_backup_array['asset_name'] = $iNames['asset_name'];
                                                    $temp_backup_array['client_name'] = $iNames['client_name'];
                                                    $temp_backup_array['app_name'] = $iNames['app_name'];
                                                    $temp_backup_array['app_type'] = $iNames['app_type'];

                                                }
                                            }

                                            $temp_backup_array['instance_id'] = isset($backup_from_schedule_history['instance_id']) ? $backup_from_schedule_history['instance_id'] : "";
                                            $temp_backup_array['message'] = isset($backup_from_schedule_history['description']) ? $backup_from_schedule_history['description'] : "";
                                            $temp_backup_array['primary_name'] = isset($backup_from_schedule_history['primary_name']) ? $backup_from_schedule_history['primary_name'] : "";
                                            $temp_backup_array['secondary_name'] = isset($backup_from_schedule_history['secondary_name']) ? $backup_from_schedule_history['secondary_name'] : "";
                                            if (isset($backup['job_start_time'])) {
                                                $temp_backup_array['start_time'] = strftime('%m/%d/%Y %r', $backup_from_schedule_history['job_start_time']);
                                                $temp_backup_array['sort_start_time'] = $backup_from_schedule_history['job_start_time'];
                                            }
                                            $temp_backup_array['failed'] = isset($backup_from_schedule_history['failed']) ? $backup_from_schedule_history['failed'] : "";
                                            $temp_backup_array['mode'] = isset($backup_from_schedule_history['type']) ? $this->functions->getBackupTypeDisplayName($backup_from_schedule_history['type']) : "";
                                            $temp_backup_array['files'] = isset($backup_from_schedule_history['files']) ? $backup_from_schedule_history['files'] : "";
                                            $temp_backup_array['size'] = isset($backup_from_schedule_history['size']) ? $backup_from_schedule_history['size'] : "";
                                            $temp_backup_array['id'] = $temp_backup_array['backup_id'] = isset($backup_from_schedule_history['backup_id']) ? $backup_from_schedule_history['backup_id'] : "N/A";
                                            $temp_backup_array['status'] = $this->jobsFunctions->getBackupStatusName($temp_backup_array['failed'], $temp_backup_array['message']);
                                            $temp_backup_array['type'] = "Backup";


                                            $jobStatus['jobs'][] = $temp_backup_array;
                                        }
                                    }
                                }
                            }
                        }
                        else
                        {
                            $jobStatus = false;
                        }

                    }
                    else
                    {
                        $jobStatus['error'] = 500;
                        $jobStatus['message'] = "Missing inputs: 'instance_id' is a required input for backup jobs that did not run";
                    }
                    break;
                default:
                    $jobStatus['error'] = 500;
                    $jobStatus['message'] = "Invalid input: 'job_type' can be ".Constants::JOBORDER_TYPE_ARCHIVE.' or '.Constants::JOBORDER_TYPE_BACKUP;
                    break;

            }
        }
        else
        {
            $jobStatus['error'] = 500;
            $jobStatus['message'] = "Missing inputs: 'id', 'job_start_time', and 'job_type' are required inputs for backup jobs.  Jobs that did not run should use an id of 0.";
        }

        return $jobStatus;
    }

    /*
     * For a given backup, return the speed (size/time) or an empty string if no rate is available.
     */
    private function getSpeed($backup) {
        $rate = "";
        if (isset($backup['elapsed_time']) && $backup['elapsed_time'] > 0 && isset($backup['size'])) {
            $rate = $backup['size'] / $backup['elapsed_time'];
            $rate = round($rate, 2);
        }
        return $rate;
    }

} //end JobsStatusFromID class
